/* ----------------------------------------------------------------------
   LAMMPS - Large-scale Atomic/Molecular Massively Parallel Simulator
   http://lammps.sandia.gov, Sandia National Laboratories
   Steve Plimpton, sjplimp@sandia.gov

   Copyright (2003) Sandia Corporation.  Under the terms of Contract
   DE-AC04-94AL85000 with Sandia Corporation, the U.S. Government retains
   certain rights in this software.  This software is distributed under 
   the GNU General Public License.

   See the README file in the top-level LAMMPS directory.
------------------------------------------------------------------------- */

#ifdef PAIR_CLASS

PairStyle(born,PairBorn)

#else

#ifndef PAIR_BORN_H
#define PAIR_BORN_H

#include "pair.h"

namespace LAMMPS_NS {

class PairBorn : public Pair {
 public:
  PairBorn(class LAMMPS *);
  virtual ~PairBorn();

  virtual void compute(int, int);
  void settings(int, char **);
  void coeff(int, char **);
  double init_one(int, int);
  void write_restart(FILE *);
  void read_restart(FILE *);
  void write_restart_settings(FILE *);
  void read_restart_settings(FILE *);
  double single(int, int, int, int, double, double, double, double &);
  void *extract(char *, int &);

 protected:
  double cut_global;
  double **cut;
  double **a,**rho,**sigma,**c, **d;
  double **rhoinv,**born1,**born2,**born3,**offset;
 
  void allocate();
};

}

#endif
#endif
